c**********************************************************************
c REORIENT.F
c**********************************************************************
c Reorient the spin axes of test particles.
c
c Input:
c  t			time
c  ntp			number of test particles
c  s(3,NTPMAX)		array of spin axes orientations /yarko/
c  dtreorient		reorientation time step
c  tau_reor(NTPMAX)	mean reorientation times
c  tau_disr(NTPMAX)	mean disruption times
c  idum			initial number for ran1
c
c Output:
c  s(3,NTPMAX)	new orientations of the spin axes /yarko/
c
c Remarks: 
c Authors:  Miroslav Broz, miroslav.broz@usa.net
c Date:  3/1/1999 

	subroutine reorient(t,ntp,dtreorient,tau_reor,tau_disr,idum,
     &    disr,reoriented,disrupted)

	include 'swift.inc'

c...    Input
        integer ntp,idum
        real*8 t,dtreorient,tau_reor(NTPMAX),tau_disr(NTPMAX)
        logical reoriented, disrupted, disr(NTPMAX)

c  input thermal parameters in common block /yarko/
      integer MAXk_seasonal             ! max # of iterations in calculation
      parameter(MAXk_seasonal=7)        ! of seasonal yarkovski effect
      integer*2 k_seasonal
      real*8 s(3,NTPMAX),lambda0(NTPMAX),Tstar40(NTPMAX),rho0(NTPMAX),
     :  lambdas0(NTPMAX),Rdash0(NTPMAX),thetas(NTPMAX),rhos0(NTPMAX),
     :  koefsin(MAXk_seasonal,NTPMAX),koefcos(MAXk_seasonal,NTPMAX)
      complex*8 psi(NTPMAX)
      common/yarko/s,psi,lambda0,Tstar40,rho0,lambdas0,Rdash0,
     :  thetas,rhos0,koefsin,koefcos,k_seasonal

c...   Internal
        integer i
        real*8 s1,coss2,sins2,p1,p2,p_reor,p_disr
        real ran1

c-----
c...  Executable code      

      disrupted = .false.
      reoriented = .false.

      do i=1,ntp

        p_disr = 1 - exp(-dtreorient/tau_disr(i))
        p1=1.0d0*ran1(idum)
        if (p2.lt.p_reor) then
          disrupted = .true.
c 2DO:  discard the i th test particle due to disruption
          disr(i) = .true.
        endif

        p_reor = 1 - exp(-dtreorient/tau_reor(i))
        p2=1.0d0*ran1(idum)
        if (p2.lt.p_reor) then
          reoriented = .true.
          s1=TWOPI*ran1(idum)
          sins2=2.0d0*ran1(idum)-1.0d0
c  sins2 = (-1.,1.) !!!
c          sins2=coss2/abs(coss2)*sqrt(1-coss2**2)
          coss2 = sqrt(1-sins2**2)
          s(1,i)=cos(s1)*coss2
          s(2,i)=sin(s1)*coss2
          s(3,i)=sins2
        endif

      enddo

      return
      end	! reorient.f
c-----------------------------------------------------------------

c**********************************************************************
      subroutine reorient_write(t,ntp,iu,fopenstat)
c**********************************************************************
c
c  t			time
c  ntp			number of test particles
c  s(3,NTPMAX)		orientations of spin axes
c  iu			unit for writing reorient.out
c  fopenstat		status of reorient.out
c
      include 'swift.inc'
      integer ntp,iu
      real*8 t
      character*80 fopenstat

c  input thermal parameters in common block /yarko/
      integer MAXk_seasonal             ! max # of iterations in calculation
      parameter(MAXk_seasonal=7)        ! of seasonal yarkovski effect
      integer*2 k_seasonal
      real*8 s(3,NTPMAX),lambda0(NTPMAX),Tstar40(NTPMAX),rho0(NTPMAX),
     :  lambdas0(NTPMAX),Rdash0(NTPMAX),thetas(NTPMAX),rhos0(NTPMAX),
     :  koefsin(MAXk_seasonal,NTPMAX),koefcos(MAXk_seasonal,NTPMAX)
      complex*8 psi(NTPMAX)
      common/yarko/s,psi,lambda0,Tstar40,rho0,lambdas0,Rdash0,
     :  thetas,rhos0,koefsin,koefcos,k_seasonal

      integer i,ierr,i1st
      data i1st/0/
      save i1st

      if (i1st.eq.0) then
        call io_open(iu,'reorient.out',fopenstat,'formatted',ierr)
        if(ierr.ne.0) then
          write(*,*) ' SWIFT ERROR: in reorient'
          write(*,*) '     Could not open reorient.out '
          call util_exit(1)
        endif
        i1st = 1
      else
        call io_open(iu,'reorient.out','append','formatted',ierr)
      endif

      write(iu,10) t
10    format(1x,1e17.10,$)
      do i=1,ntp - 1
        write(iu,20) s(1,i),s(2,i),s(3,i)
20      format(1x,3(1x,1f10.8),$)
      enddo
      write(iu,30) s(1,ntp),s(2,ntp),s(3,ntp)
30    format(1x,3(1x,1f10.8))

      close(unit = iu)

      return
      end	! reorient_write

c**********************************************************************
      subroutine disrupt_write(t,ntp,disr,iu,fopenstat)
c**********************************************************************
c
c  t			time
c  ntp			number of test particles
c  disr(NTPMAX)		disruption flag for test particles
c  iu			unit for writing disrupt.out
c  fopenstat		status of disrupt.out
c
      include 'swift.inc'
      integer ntp,iu
      real*8 t
      logical disr(NTPMAX)
      character*80 fopenstat

      integer i,ierr,i1st
      data i1st/0/
      save i1st

      if (i1st.eq.0) then
        call io_open(iu,'disrupt.out',fopenstat,'formatted',ierr)
        if(ierr.ne.0) then
          write(*,*) ' SWIFT ERROR: in reorient'
          write(*,*) '     Could not open disrupt.out '
          call util_exit(1)
        endif
        i1st = 1
      else
        call io_open(iu,'disrupt.out','append','formatted',ierr)
      endif

      write(iu,10) t,(disr(i),i=1,ntp)
10    format(1x,1e17.10,1000(1x,l1))

      close(unit = iu)

      return
      end	! disrupt_write

